from queue import Queue
from threading import Thread
from code_domain_emissary.emissary import Emissary

class Extractor:

    def __init__(self, q):
        self._input_q = q
        self._transcribe_thread = Thread(target=self._watch_q)
        self._transcribe_thread.setDaemon(True)
        self._run = False
        self.code_emissary = Emissary()
        self.output_q = Queue()

    def observe(self):
        self._run = True
        self._transcribe_thread.start()

    def ignore(self):
        self._run = False

    def _watch_q(self):
        while self._run:
            text = self._input_q.get(True)
            # send to NLP and respond with processed intents
            result = self.code_emissary.process(text)
            self.output_q.put(result)
            # print(result)



import time
from threading import Timer
from unittest import TestCase
from unittest import mock
from ..listener import Listener
import os
import speech_recognition as sr


class ListenerTest(TestCase):

    def setUp(self) -> None:
        pass

    def tearDown(self) -> None:
        pass

    def test_successful_strategy_mapping(self):
        with mock.patch('speech_recognition.Microphone', return_value=object):
            try:
                _ = Listener(strategy="sphinx")
                assert True
            except KeyError:
                self.fail()
            except Exception:
                self.fail()

    def test_erroneous_strategy_mapping(self):
        try:
            _ = Listener(strategy="testException")
            self.fail()
        except KeyError:
            self.assertRaises(KeyError)
        except Exception:
            self.fail()

    def test_pyaudio_not_installed(self):
        with mock.patch('speech_recognition.Microphone') as mocked_mic:
            mocked_mic.side_effect = AttributeError("test")
            try:
                _ = Listener()
                self.fail()
            except AttributeError as err:
                self.assertEqual("You'll need to install PyAudio first: https://people.csail.mit.edu/hubert/pyaudio/", str(err))
            except Exception:
                self.fail()

    def test_no_mic_found(self):
        with mock.patch('speech_recognition.Microphone', return_value=None):
            l = Listener()
            try:
                l.listen()
                l.stop()
                self.fail()
            except NameError as err:
                self.assertEqual("Could not find a suitable microphone.", str(err))
            except Exception:
                self.fail()

    def test_could_not_parse_text(self):
        with mock.patch('speech_recognition.Microphone') as mocked_mic:
            mocked_mic.return_value.__enter__ = lambda x: time.sleep(.5)
            rel_path = os.path.join(os.getcwd(), "audio-files/harvard.wav")
            harvard = sr.AudioFile(rel_path)
            os.system = mock.MagicMock()
            l = Listener()
            with harvard as source:
                audio = l.recognizer.record(source)
            l.recognizer.listen = mock.MagicMock(return_value=audio)
            l.transcribe = mock.Mock(side_effect=sr.UnknownValueError("test"))
            s = Timer(1.0, l.stop)
            s.start()
            l.listen()
            assert os.system.called



import queue
import time
from unittest import TestCase
from ..extractor import Extractor


class ExtractorTest(TestCase):

    def setUp(self) -> None:
        self._test_input_q = queue.Queue()
        self._extractor = Extractor(self._test_input_q)
        pass

    def tearDown(self) -> None:
        pass

    def test_successful_start(self):
        self._extractor.observe()
        self._test_input_q.put("test!")
        time.sleep(.1)
        assert not self._extractor.output_q.empty()

    def test_successful_stop(self):
        self._extractor.observe()
        self._extractor.ignore()
        self._test_input_q.put("test!")
        _ = self._extractor.output_q.get(True)
        self._test_input_q.put("test again!")
        time.sleep(.1)
        assert self._extractor.output_q.empty()



import queue
import threading

import speech_recognition as sr
import os

STRATEGY_MAP = {
    "sphinx": "recognize_sphinx",
    "bing": "recognize_bing",
    "google": "recognize_google",
    "google_cloud": "recognize_google_cloud",
    "houndify": "recognize_houndify",
    "ibm": "recognize_ibm",
    "wit": "recognize_wit"
}

STRATEGY_LIST = ('sphinx', 'bing', 'google', 'google_cloud', 'houndify', 'ibm', 'wit')


class Listener:

    def __init__(self, strategy="google", credentials=None):
        self._available_strategies = STRATEGY_LIST
        self._set_strategy(strategy)
        self._credentials = credentials
        self._run = False
        self._spoken_q = queue.Queue()
        self.output_q = queue.Queue()
        try:
            self.mic = sr.Microphone()
        except AttributeError:
            raise AttributeError("You'll need to install PyAudio first: https://people.csail.mit.edu/hubert/pyaudio/")

    def _set_strategy(self, chosen_strategy):
        self.recognizer = sr.Recognizer()
        self.recognizer.pause_threshold = 0.5
        try:
            strategy = STRATEGY_MAP[chosen_strategy]
            self.transcribe = self.recognizer.__getattribute__(strategy)
        except KeyError:
            raise KeyError("You'll need to select an available strategy: {}.  Note that the default is `google`.".format(self._available_strategies))

    def _activate(self):
        try:
            assert self.mic is not None
            with self.mic as source:
                sounds = self.recognizer.listen(source)
                self._spoken_q.put(sounds)
            while self._run:
                self._activate()
        except AssertionError:
            raise NameError("Could not find a suitable microphone.")

    def _parse(self):
        while self._run:
            audio_input = self._spoken_q.get(True)
            try:
                speech_text = self.transcribe(audio_input)
                self.output_q.put(speech_text)
            except sr.UnknownValueError:
                self._notify_error("There was a problem", "Could not process your speech into text")

    def _notify_error(self, title, err_text):
        os.system("""
        osascript -e 'display notification "{}" with title "{}"'
        """.format(err_text, title))

    def listen(self):
        self._run = True
        parse_thread = threading.Thread(target=self._parse)
        parse_thread.setDaemon(True)
        parse_thread.start()
        self._activate()

    def stop(self):
        self._run = False



from setuptools import setup, find_packages

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

requirements = ["PyAudio==0.2.11", "SpeechRecognition==3.8.1"]

setup(
    name="vocalist",
    version="0.0.1.dev1",
    author="Colin Lacy",
    author_email="colinjlacy@gmail.com",
    description="A package to convert speech to long-form work-flows, using plugable trained NLP models",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)
